#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>

#define NOF_ELEM 10000
#define STR_LEN 64

typedef char fixed_string[STR_LEN];

#define BEGIN_MEASURE { \
    struct timeval begin; \
    gettimeofday(&begin, NULL);

#define END_MEASURE(title) \
    struct timeval end; \
    gettimeofday(&end, NULL); \
    printf("%12s: %lu\n", title, (end.tv_sec * 1000000 + end.tv_usec) - (begin.tv_sec * 1000000 + begin.tv_usec)); \
}

const char * gen_str() {
    static fixed_string s;
    for (size_t i = 0; i < sizeof(fixed_string); ++i) {
        s[i] = rand() % 25 + 65;
    }
    return s;
}

int opt_memcmp(const void *s1, const void *s2, size_t n);
int my_memcmp(const void *s1, const void *s2, size_t n);

int main()
{
    clock_t begin, end;

    fixed_string arr1[NOF_ELEM][STR_LEN];
    char crap;
    fixed_string arr2[NOF_ELEM][STR_LEN];
    int r1[NOF_ELEM];
    int r2[NOF_ELEM];
    int r3[NOF_ELEM];

    for (size_t i = 0; i < NOF_ELEM; ++i) {
        strncpy((char *)arr1[i], gen_str(), sizeof(*arr1)-1);
        strncpy((char *)arr2[i], gen_str(), sizeof(*arr2)-1);
    }

    /* memcmp */
    BEGIN_MEASURE
    for (size_t i = 0; i < NOF_ELEM; ++i) {
        r1[i] = memcmp(arr1[i], arr2[i], sizeof(*arr1));
    }
    END_MEASURE("memcmp")

    /* my_memcmp */
    BEGIN_MEASURE
    for (size_t i = 0; i < NOF_ELEM; ++i) {
        r2[i] = my_memcmp(arr1[i], arr2[i], sizeof(*arr1));
    }
    END_MEASURE("my_memcmp")

    /* opt_memcmp */
    BEGIN_MEASURE
    for (size_t i = 0; i < NOF_ELEM; ++i) {
        r3[i] = opt_memcmp(arr1[i], arr2[i], sizeof(*arr1));
    }
    END_MEASURE("opt_mycmp")

    for (size_t i = 0; i < NOF_ELEM-1; ++i) {
        assert(r1[i] == r2[i]);
        assert(r1[i] == r3[i]);
    }
}
